/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 * Note(s): possible defines set in "options for target - C/C++ - Define"
 *            __USE_LCD    - enable Output on GLCD
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2018 Arm Limited

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "RTE_Components.h"             // Component selection
#include  CMSIS_device_header           // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons
#include "Board_GLCD.h"                 // ::Board Support:Graphic LCD
#include "GLCD_Config.h"                // Keil.V2M-MPS2+::Board Support:Graphic LCD

extern GLCD_FONT     GLCD_Font_16x24;

#if defined RTE_CMSIS_RTOS2_RTX5
/* stack size must be multiple of 8 Bytes */
#define APP_MAIN_STK_SZ (1024U)
extern uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
       uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
extern const osThreadAttr_t app_main_attr;
       const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};
#endif

static volatile uint32_t delay_val = 500U;

static osThreadId_t tid_thrLED;                /* Thread id of thread: LED */
static osThreadId_t tid_thrBUT;                /* Thread id of thread: BUT */
#ifdef __USE_LCD
static osThreadId_t tid_thrLCD;                /* Thread id of thread: LCD */
static osMutexId_t  mid_mutLCD;                /* Mutex  id of mutex:  LCD */
#endif

/*----------------------------------------------------------------------------
  thrLED: blink LED
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrLED(void *argument) {
  uint32_t ledMax    = LED_GetCount();
  uint32_t ledNum    = 0;

  (void)argument;

  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_On(ledNum);                                           /* Turn specified LED on */
#ifdef __USE_LCD
    osMutexAcquire(mid_mutLCD, osWaitForever);
    GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
    GLCD_SetForegroundColor(GLCD_COLOR_GREEN);
    GLCD_DrawChar((9U+ledNum)*16U, 6U*24U, 0x80+1U);          /* Circle full */
    osMutexRelease(mid_mutLCD);
#endif

    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_Off(ledNum);                                          /* Turn specified LED off */
#ifdef __USE_LCD
    osMutexAcquire(mid_mutLCD, osWaitForever);
    GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
    GLCD_SetForegroundColor(GLCD_COLOR_LIGHT_GREY);
    GLCD_DrawChar((9U+ledNum)*16U, 6U*24U, 0x80+0U);          /* Circle empty */
    osMutexRelease(mid_mutLCD);
#endif

    ledNum++;                                                 /* Change LED number */
    if (ledNum >= ledMax) {
      ledNum = 0;                                             /* Restart with first LED */
    }
  }

}

/*----------------------------------------------------------------------------
  thrBUT: check button state
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrBUT(void *argument) {
  uint32_t butState = 0;;
  uint32_t butMax   = Buttons_GetCount();
  uint32_t butMask  = (1U << butMax) - 1;
#ifdef __USE_LCD
  uint32_t butNum   = 0;
#endif

  (void)argument;

  for (;;) {
    do {
      butState = Buttons_GetState();
#ifdef __USE_LCD
      for (butNum = 0; butNum < butMax; butNum++) {
        if (butState & (1UL << butNum)) {
          osMutexAcquire(mid_mutLCD, osWaitForever);
          GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
          GLCD_SetForegroundColor(GLCD_COLOR_BLACK);
          GLCD_DrawChar((9U+butNum)*16U, 7U*24U, 0x80+1U);    /* Circle full */
          osMutexRelease(mid_mutLCD);
        } else {
          osMutexAcquire(mid_mutLCD, osWaitForever);
          GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
          GLCD_SetForegroundColor(GLCD_COLOR_LIGHT_GREY);
          GLCD_DrawChar((9U+butNum)*16U, 7U*24U, 0x80+0U);    /* Circle empty */
          osMutexRelease(mid_mutLCD);
        }
      }
#endif
    } while (butState & butMask);                             /* Wait while holding USER button */

    osThreadFlagsSet(tid_thrLED, 0x0001U);
    osDelay(delay_val);
  }

}

#ifdef __USE_LCD
/*----------------------------------------------------------------------------
  thrLCD: update GLCD
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrLCD(void *argument) {
  uint32_t FGColor = GLCD_COLOR_WHITE;
  uint32_t ledMax = LED_GetCount() - 1;
  uint32_t butMax = Buttons_GetCount();
  uint32_t i;

  (void)argument;

  osMutexAcquire(mid_mutLCD, osWaitForever);
  GLCD_SetFont(&GLCD_Font_16x24);
  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_ClearScreen();

  GLCD_SetBackgroundColor(GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor(GLCD_COLOR_BLACK);
  GLCD_DrawString(2U*16U, 6U*24U, "LED:");
  GLCD_DrawString(2U*16U, 7U*24U, "KEY:");

  GLCD_SetForegroundColor (GLCD_COLOR_LIGHT_GREY);
  for (i = 0; i <= ledMax; i++) {
    GLCD_DrawChar((9U+i)*16U, 6U*24U, 0x80+0U);               /* Circle empty */
  }
  for (i = 0; i <  butMax; i++) {
    GLCD_DrawChar((9U+i)*16U, 7U*24U, 0x80+0U);               /* Circle empty */
  }
  osMutexRelease(mid_mutLCD);

  for (;;) {
    if (FGColor == GLCD_COLOR_WHITE) {
      FGColor = GLCD_COLOR_RED;
    } else {
      FGColor = GLCD_COLOR_WHITE;
    }
    osMutexAcquire(mid_mutLCD, osWaitForever);
    GLCD_SetBackgroundColor(GLCD_COLOR_BLUE);
    GLCD_SetForegroundColor(FGColor);
    GLCD_DrawString(0U*16U, 0U*24U, "   SAM3S-EK Demo    ");
    GLCD_DrawString(0U*16U, 1U*24U, "      Blinky        ");
    GLCD_DrawString(0U*16U, 2U*24U, "   www.keil.com     ");
    osMutexRelease(mid_mutLCD);

    osDelay(delay_val * 4);
  }

}
#endif

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *argument);
__NO_RETURN void app_main (void *argument) {

  (void)argument;

  LED_Initialize ();
  Buttons_Initialize ();
#ifdef __USE_LCD
  GLCD_Initialize ();
#endif

#ifdef __USE_LCD
  mid_mutLCD = osMutexNew(NULL);                            /* create LCD mutex */
  if (mid_mutLCD == NULL) { /* add error handling */ }

  tid_thrLCD = osThreadNew (thrLCD, NULL, NULL);            /* create LCD thread */
  if (tid_thrLCD == NULL) { /* add error handling */ }
#endif

  tid_thrLED = osThreadNew (thrLED, NULL, NULL);            /* create LED thread */
  if (tid_thrLED == NULL) { /* add error handling */ }

  tid_thrBUT = osThreadNew (thrBUT, NULL, NULL);            /* create BUT thread */
  if (tid_thrBUT == NULL) { /* add error handling */ }

  osThreadExit ();
//  for (;;) {}
}
